/*
 * A simple read-eval-print loop for the scheme interpreter
 */
#include <stdio.h>
#include <string.h>

#include "scheme.h"


int main(int argc, char *argv[])
{
	int i;
	context *ctx;
	
	scheme_init();
	
	ctx = create_context();
	
	if (argc == 1) {
		/* No arguments given, start read-eval-print loop */
		for (;;) {
			/* Read */
			char line[256];
			
			if (feof(stdin))
				return 0;
			
			(void)memset(line, 0, sizeof(line));
			(void)fgets(line, (int)sizeof(line), stdin);
			
			if (line[0] != '\0' && line[0] != '\n') {
				/* Evaluate */
				obj *o = eval_string(ctx, line);
				
				if (ctx->error[0] != 0) {
					fprintf(stderr, "%s", ctx->error);
					
					continue;
				}
				
				/* Print */
				display(ctx, stdout, o);
				printf("\n");
			}
		}
	}
	
	/* Otherwise execute the given files in order */
	for (i = 1; i < argc; i++) {
		(void)eval_file(ctx, argv[i]);
		
		if (ctx->error[0] != 0) {
			fprintf(stderr, "%s", ctx->error);
		}
	}
	
	destroy_context(ctx);
	
	return 0;
}
